package temportalist.origin.foundation.common

import net.minecraftforge.fml.client.event.ConfigChangedEvent.OnConfigChangedEvent
import net.minecraftforge.fml.common.Mod
import net.minecraftforge.fml.common.event.{FMLInitializationEvent, FMLPostInitializationEvent, FMLPreInitializationEvent, FMLServerStartingEvent}
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent
import org.apache.logging.log4j.{LogManager, Logger}
import temportalist.origin.foundation.common.modTraits._
import temportalist.origin.foundation.common.network.NetworkMod
import temportalist.origin.foundation.common.registers.{Register, RegisterPhase}

import scala.reflect.classTag
import scala.util.Sorting

/**
  *
  * Created by TheTemportalist on 4/9/2016.
  *
  * @author TheTemportalist
  */
abstract class IMod extends NetworkMod with Options with Registry {

	private var logger: Logger = _

	final def log(format: String, data: AnyRef*): Unit = this.logger.info(format, data:_*)

	def getProxy: IProxy

	def getRegisters: Seq[Register] = Seq()

	private var sortedRegisters: Array[Register] = _

	protected def preInitialize(event: FMLPreInitializationEvent): Unit = {
		this.logger = event.getModLog
		this.handleConfiguration(event)

		this.sortedRegisters = Sorting.stableSort(
			this.getRegisters)(classTag[Register], Register.Order)

		for (phase <- RegisterPhase.PREINIT_ORDER) {
			for (reg <- this.sortedRegisters) {
				if (phase == RegisterPhase.PREINIT_ORDER.head)
					reg.setMod(this.getDetails)
				Register.doRegistration(reg, phase, this.getDetails, event)
			}
		}

		this.registerHandler(this, this.getProxy)
		this.getProxy.preInit()
		this.registerGuiHandler(this, this.getProxy)

	}

	protected def initialize(event: FMLInitializationEvent): Unit = {
		this.sortedRegisters.foreach(reg => Register.doRegistration(reg, RegisterPhase.INIT, this.getDetails, event))
		this.getProxy.register()
	}

	protected def postInitialize(event: FMLPostInitializationEvent): Unit = {
		this.getProxy.postInit()

		this match {
			case tabContainer: IHasTabs =>
				tabContainer.initTabs()
			case _ =>
		}
	}

	@SubscribeEvent
	def onConfigurationChanged(event: OnConfigChangedEvent): Unit = this.onConfigChange(event)

	@Mod.EventHandler
	def serverStarting(event: FMLServerStartingEvent): Unit = {
		this match {
			case hazCommands: IHasCommands =>
				for (command <- hazCommands.getCommands)
					event.registerServerCommand(command)
			case _ =>
		}
	}

}
